#include "mpi.h"
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

#define wait_for_gdb() do { int __wait_for_gdb = 0; fprintf(stderr, "Process %d waiting...\n", getpid()); while(__wait_for_gdb == 0) { sleep(1); }; } while(0)
#define checkpoint() do { fprintf(stderr, "%d:%s:%d\n", myid, __FILE__, __LINE__); } while(0)

int main(int argc, char *argv[]) {
  int done = 0, n, myid, numprocs, i, rc;
  double PI25DT = 3.141592653589793238462643;
  double mypi, pi, h, sum, x, a;
  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD, &myid);

  while (!done) {
    if (myid == 0) {
      // wait_for_gdb();
      // printf("Enter the number of intervals: (0 quits) ");
      // scanf("%d", &n);
      n = rand() % 1024;
    }
    MPI_Bcast(&n, 1, MPI_INT, /* rank of the sender */ 0, MPI_COMM_WORLD);
    if (n == 0)
      break;
    h = 1.0 / (double)n;
    sum = 0.0;
    for (i = myid + 1; i <= n; i += numprocs) {
      x = h * ((double)i - 0.5);
      sum += 4.0 / (1.0 + x * x);
    }
    mypi = h * sum;
    MPI_Reduce(&mypi, &pi, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
    if (myid == 0)
      printf("After %d iterations, pi is approximately %.16f, Error is %.16f\n", n, pi,
             fabs(pi - PI25DT));
  }
  MPI_Finalize();
  return 0;
}
